const { app, BrowserWindow, dialog, Menu } = require('electron');
const fs = require("fs");
const {ipcMain} = require("electron")

let mainWindow = null;
let save_data = {
  file_path: null
}
let menu_template = [
  {
    label: "File",
    submenu: [
      {
        label: "Save",
        accelerator: "Ctrl+S",
        click: function () {
          mainWindow.webContents.send("save", false)
        }
      },
      {
        label: "Save As",
        accelerator: "Ctrl+Shift+S",
        click: function () {
          mainWindow.webContents.send("save", true)
        }
      },
      {
        label: "Open",
        accelerator: "Ctrl+O",
        click: function () {
          let options = {
            //Placeholder 1
            title: "Open website data",

            //Placeholder 2
            defaultPath : app.getPath("documents"),

            //Placeholder 3
            filters :[
              {name: 'json', extensions: ['json']},
              {name: 'All Files', extensions: ['*']}
            ]
          }
          let file = dialog.showOpenDialogSync(options);
          if (file) {
            save_data.file_path = file[0];
            mainWindow.webContents.send("open", save_data.file_path);
          }
        }
      },
      {
        label: "Exit",
        click: function () {
          app.quit();
        }
      }
    ]
  }
]

function sanitizeName(table_name) {
  return table_name.toLowerCase().replace(/ /g, "")
}

ipcMain.on("save", (event, data, new_file) => {
  let default_path = app.getPath("documents");
  if (save_data.file_path) {
    default_path = save_data.file_path;
  }
  let options = {
    //Placeholder 1
    title: "Save website data",

    //Placeholder 2
    defaultPath : default_path,

    //Placeholder 3
    filters :[
      {name: 'json', extensions: ['json']},
      {name: 'All Files', extensions: ['*']}
    ]
  }

  if (new_file || !save_data.file_path) {
    let path = dialog.showSaveDialogSync(options);
    if (path) {
      save_data.file_path = path;
    }
  }
  if (save_data.file_path) {
    fs.writeFileSync(app.getPath("userData")+"\\data.json", JSON.stringify(save_data))
    fs.writeFileSync(save_data.file_path, JSON.stringify(data, null, 4));
    let name = save_data.file_path.match(/.+\\([^\\]+)/)[1];
    let nav_content = save_data.file_path.replace(".json", "-nav-content.html");
    let tab_content = save_data.file_path.replace(".json", "-tab-content.html");
    let tabs = "";
    let nav = "";
    for (let table of data) {
      if (!table["hidden"]) {
        tabs += `<div class="tab-pane fade" id="${sanitizeName(table["name"])}" role="tabpanel" aria-labelledby="${sanitizeName(table["name"])}-tab"><h3 id="${sanitizeName(table["name"])}-header">${table["name"]} (<span id="${sanitizeName(table["name"])}-completed"></span>/<span id="${sanitizeName(table["name"])}-total"></span>)</h3><table id="${sanitizeName(table["name"])}-table" class="table table-hover"></table></div>`;
        nav += `<li class="nav-item"><a class="nav-link" id="${sanitizeName(table["name"])}-tab" data-toggle="tab" href="#${sanitizeName(table["name"])}" role="tab" aria-controls="${sanitizeName(table["name"])}" aria-selected="false">${table["name"]}</a></li>`
      }
    }
    fs.writeFileSync(nav_content, nav);
    fs.writeFileSync(tab_content, tabs);
  }
})

// Handle creating/removing shortcuts on Windows when installing/uninstalling.
if (require('electron-squirrel-startup')) { // eslint-disable-line global-require
  app.quit();
}

const createWindow = () => {
  // Create the browser window.
  mainWindow = new BrowserWindow({
    width: 800,
    height: 600,
    webPreferences: {
      nodeIntegration: true
    }
  });

  let menu = Menu.buildFromTemplate(menu_template);
  Menu.setApplicationMenu(menu);

  // and load the index.html of the app.
  mainWindow.loadFile('src/index.html');

  // Open the DevTools.
  mainWindow.webContents.openDevTools();
};

// This method will be called when Electron has finished
// initialization and is ready to create browser windows.
// Some APIs can only be used after this event occurs.
app.on('ready', createWindow);

// Quit when all windows are closed, except on macOS. There, it's common
// for applications and their menu bar to stay active until the user quits
// explicitly with Cmd + Q.
app.on('window-all-closed', () => {
  if (process.platform !== 'darwin') {
    app.quit();
  }
});

app.on('activate', () => {
  // On OS X it's common to re-create a window in the app when the
  // dock icon is clicked and there are no other windows open.
  if (BrowserWindow.getAllWindows().length === 0) {
    createWindow();
  }
});

// In this file you can include the rest of your app's specific main process
// code. You can also put them in separate files and import them here.
