const {ipcRenderer} = require("electron")
let data;
let selected = null;
let current_table = null;
let current_col = null;
let current_row = null;
let uid_regex = /(.+)-row(\d+)/;
$.ajax({
	dataType: "json",
	url: "database.json",
	cache: false,
	success: function(json, textStatus, jqXHR ) {
		data = json;
		refreshTables();
		showTable(sanitizeName(data[0]["name"]));
	}
});

function sanitizeName(table_name) {
	return table_name.toLowerCase().replace(/ /g, "")
}

function replaceLast(str, str1, str2) {
	let last_index = str.lastIndexOf(str1);
	str = str.slice(0, last_index) + str.slice(last_index).replace(str1, str2);
	return str;
}

function makeTables() {
	for (let table of data) {
		let name = sanitizeName(table["name"]);
		let table_index = data.indexOf(table);

		// create nav element
		$("nav").append(`<button id="${name}-nav" class="w3-bar-item w3-button tablink" style="white-space: nowrap" onclick="showTable('${name}')" draggable="true" ondrop='tableDrop(event, ${table_index})' ondragstart='tableDrag(event, ${table_index})' ondragover='allowDrop(event)'>${table["name"]}</button>`);
		let table_elm = $("<table class='editor-table'></table>");
		table_elm.attr("id", name+"-table");

		let cols = table["col-def"]

		let tr = $("<tr></tr>");
		let td = $("<td></td>");
		td.css("border", "none");
		if (data.length > 1) {
			td.html(`<button class="w3-button w3-red" onclick="deleteTable('${table["name"]}')">Delete Table</button>`);
		}
		tr.append(td);

		let index = 0;
		for (let col of cols) {
			let id = name+"-"+index.toString();
			let th = $(`<th class="w3-display-container" onclick="selectCell('${table["name"]}', '${col["title"]}', '${col["field"]}', '${id}')" draggable="true" ondrop="colDrop(event, '${name}', ${index})" ondragstart="drag(event, ${index})" ondragover="allowDrop(event)" ondblclick="startEditHeader('${id}')"></th>`);
			th.attr("id", id);
			th.append(`<span style="margin-right: 44px; white-space: nowrap">${col.title}</span>`);
			th.append(`<span class="w3-button w3-display-right w3-hover-red" onclick="deleteCol('${table["name"]}', ${index})">&times;</span>`);
			tr.append(th);
			index++;
		}

		td = $("<td></td>");
		td.css("border", "none");
		td.html(`<button class="w3-button w3-green" onclick="addColumn('${table["name"]}')">Add Column</button>`);
		tr.append(td);
		table_elm.append(tr);
		for (let row of table["rows"]) {
			let index = table["rows"].indexOf(row);
			tr = $(`<tr id="${row["uid"]}" ondrop="rowDrop(event, '${name}', ${index})" ondragover="allowDrop(event)"></tr>`);

			td = $("<td></td>");
			td.css("border", "none");
			td.append(`<button class="w3-button w3-tiny w3-red" onclick="deleteRow('${table["name"]}', ${index})">×</button>`);
			td.append(`<div style="display: inline; float: right; font-size: 2em" draggable="true" ondragstart="rowDrag(event, '${index}')">☰</div>`);
			tr.append(td);

			for (let col of cols) {
				let uid = row["uid"]+"-"+col["field"];
				td = $(`<td id="${uid}" onclick="selectCell('${table["name"]}', '${col["title"]}', '${col["field"]}', '${uid}')" ondblclick="startEditCell('${uid}')"></td>`);
				td.html(row[col["field"]]);
				// td.click(function (){
				//   selectCell(table["name"], col["title"], col["field"], index);
				// });
				tr.append(td);
			}
			table_elm.append(tr);
		}

		tr = $("<tr></tr>");
		td = $("<td></td>");
		td.css("border", "none");
		td.html(`<button class="w3-button w3-green" onclick="addRow('${name}')">Add Row</button>`);
		tr.append(td);
		table_elm.append(tr);

		// table_elm.attr("class", "table table-hover");
		$("#tables").append(table_elm);
	}
	$("nav").append("<button class='w3-button w3-green' onclick='addTable()'>+</button>");
}

function removeTables() {
	$("#nav").empty();
	$("#tables").empty();
}

function refreshTables() {
	removeTables();
	makeTables();
	if (current_table != null) {
		showTable(current_table);
	}
}

function addColumn(table_name) {
	for (let table of data) {
		if (table["name"] == table_name) {
			let def_name = "Column";
			let names = [];
			for (let i of table["col-def"]) {
				names.push(i["title"]);
			}

			let num = 1;
			let name = `${def_name} ${num}`;
			for (let i in names) {
				if (names.includes(name)) {
					num++;
					name = `${def_name} ${num}`;
				}
			}

			let field = name.toLowerCase().replace(" ", "");
			table["col-def"].push({
				"field": field,
				"title": name
			})

			for (let row of table["rows"]) {
				row[field] = "";
			}
		}
	}
	refreshTables();
}

function addRow(table_name) {
	for (let table of data) {
		if (sanitizeName(table["name"]) == table_name) {
			let row = {};
			for (let col of table["col-def"]) {
				row[col["field"]] = "";
			}

			let uids = [];
			for (let i of table["rows"]) {
				uids.push(i["uid"]);
			}

			let index = 0;
			let uid = `${table_name}-row${index}`
			for (let i in uids) {
				if (uids.includes(uid)) {
					index++;
					uid = `${table_name}-row${index}`;
				}
			}
			row["uid"] = uid;
			table["rows"].push(row);
		}
	}
	refreshTables();
}

function addTable() {
	let def_name = "Table";
	let names = [];
	for (let table of data) {
		names.push(table["name"]);
	}

	let num = 1;
	let name = `${def_name} ${num}`;
	for (let i in names) {
		if (names.includes(name)) {
			num++;
			name = `${def_name} ${num}`;
		}
	}

	data.push({
		name: name,
		filter: false,
		hidden: false,
		"col-def": [
			{
				"field": "column1",
				"title": "Column 1",
				"sortable": false,
				"selectable": false
			},
			{
				"field": "column2",
				"title": "Column 2",
				"sortable": false,
				"selectable": false
			}
		],
		rows: [
			{
				"uid": sanitizeName(name)+"-row0",
				"column1": "",
				"column2": "",
			}
		]
	})
	refreshTables();
}

function selectCell(table_name, title, field, cell_id) {
	let elm = $("#"+cell_id);
	if (elm.is("th")) {
		$("#cell-text").prop("disabled", true);
		$("#cell-text").val("");
	}
	else {
		$("#cell-text").prop("disabled", false);
	}
	selected = {table_name:table_name,title:title,field:field, cell:cell_id};
	current_col = {"title":title, "field":field};
	$("td").removeClass("w3-green");
	$("th").removeClass("w3-green");
	elm.addClass("w3-green");
	for (let table of data) {
		if (table["name"] == table_name) {
			$("#table-name").val(table_name);
			$("#table-hidden").prop("checked", table["hidden"]);
			$("#col-name").val(title);
			for (let col of table["col-def"]) {
				if (col["title"] == title) {
					$("#col-sortable").prop("checked", col["sortable"]);
					$("#col-selectable").prop("checked", col["selectable"]);
				}
			}
			for (let row of table["rows"]) {
				if (row["uid"]+`-${field}` == cell_id) {
					$("#cell-text").val(row[field]);
				}
			}
		}
	}
}

function save() {
	if (selected != null) {
		let table_name = $("#table-name").val();
		let table_hidden = $("#table-hidden").prop("checked");
		let title = $("#col-name").val();
		let field = $("#col-name").val().toLowerCase().replace(" ", "");
		let sortable = $("#col-sortable").prop("checked");
		let selectable = $("#col-selectable").prop("checked");
		let cell_text = $("#cell-text").val();
		for (let table of data) {
			if (table["name"] == selected["table_name"]) {
				if (table["name"] != table_name) {
					let table_exsits = false;
					for (let other_table of data) {
						if (other_table["name"] == table_name) {
							table_exsits = true;
						}
					}
					if (!table_exsits && table_name.length > 0) {
						selected["cell"] = selected["cell"].replace(sanitizeName(table["name"]), sanitizeName(table_name));
						table["name"] = table_name;
						for (let row of table["rows"]) {
							let match = row["uid"].match(uid_regex);
							row["uid"] = `${sanitizeName(table_name)}-row${match[2]}`;
						}
						$("#table-name").removeClass("w3-red");
					} else {
						$("#table-name").addClass("w3-red");
						return;
					}
				}
				table["hidden"] = table_hidden;
				for (let col of table["col-def"]) {
					if (col["field"] == selected["field"]) {
						if (col["field"] != field) {
							let col_exists = false;
							for (let other_col of table["col-def"]) {
								if (other_col["field"] == field) {
									col_exists = true;
								}
							}

							if (!col_exists && field.length > 0) {
								for (let row of table["rows"]) {
									console.log(field);
									console.log(col["field"]);
									row[field] = row[col["field"]];
									delete row[col["field"]];
								}
								selected["cell"] = replaceLast(selected["cell"], col["field"], field);
								col["title"] = title;
								col["field"] = field;
								$("#col-name").removeClass("w3-red");
							} else {
								$("#col-name").addClass("w3-red");
								return;
							}
						}
						col["sortable"] = sortable;
						col["selectable"] = selectable;
					}
				}
				for (let row of table["rows"]) {
					if (row["uid"]+"-"+field == selected["cell"]) {
						row[selected["field"]] = cell_text;
					}
				}
			}
		}
		selected = {table_name:table_name,title:title,field:field,row_index:selected["row_index"], cell:selected["cell"]};
		current_table = sanitizeName(table_name);
		current_col = {"title":title, "field":field};
		current_row = selected["row_index"];
	}
	refreshTables();
}

function deleteTable(table_name) {
	for (let i=0; i<data.length; i++) {
		if (data[i]["name"] == table_name) {
			data.splice(i, 1);
		}
	}
	refreshTables();
}

function deleteRow(table_name, index) {
	for (let table of data) {
		if (table["name"] == table_name) {
			table["rows"].splice(index, 1);
		}
	}
	refreshTables();
}

function deleteCol(table_name, index) {
	for (let table of data) {
		if (table["name"] == table_name) {
			table["col-def"].splice(index, 1);
		}
	}
	refreshTables();
}

function moveRow(table_name, old_index, new_index) {
	for (let table of data) {
		if (sanitizeName(table["name"]) == table_name) {
			let row = table["rows"][old_index];

			table["rows"].splice(old_index, 1);
			table["rows"].splice(new_index, 0, row);
		}
	}
	selectCell(selected["table_name"], selected["title"], selected["field"], selected["cell"]);
	refreshTables();
}

function moveCol(table_name, old_index, new_index) {
	for (let table of data) {
		if (sanitizeName(table["name"]) == table_name) {
			let col = table["col-def"][old_index];
			table["col-def"].splice(old_index, 1);
			table["col-def"].splice(new_index, 0, col);
		}
	}
	refreshTables();
}

function moveTable(old_index, new_index) {
	let table = data[old_index];
	data.splice(old_index, 1);
	data.splice(new_index, 0, table);
	refreshTables();
}

function drag(event, old_index) {
	event.dataTransfer.setData("text", old_index);
}

function rowDrag(event, old_index) {
	event.dataTransfer.setData("text", old_index);
}

function tableDrag(event, table_index) {
	event.dataTransfer.setData("text", table_index.toString());
}

function allowDrop(event) {
	event.preventDefault();
}

function colDrop(event, table_name, new_index) {
	event.preventDefault();
	let old_index = event.dataTransfer.getData("text");
	moveCol(table_name, parseInt(old_index), new_index);
}

function rowDrop(event, table_name, new_index) {
	event.preventDefault();
	let old_index = event.dataTransfer.getData("text");
	moveRow(table_name, parseInt(old_index), new_index);
}

function tableDrop(event, new_index) {
	event.preventDefault();
	let old_index = event.dataTransfer.getData("text");
	moveTable(parseInt(old_index), new_index);
}

function showTable(table_name) {
	let reset = false;
	if (current_table != table_name) {
		current_table = table_name;
		reset = true;
	}

	let tables = $(".editor-table");
	tables.css("display", "none");
	$(`#${table_name}-table`).css("display", "block");
	let tabs = $(".tablink");
	tabs.removeClass("w3-grey");
	$(`#${table_name}-nav`).addClass("w3-grey");
	for (let table of data) {
		if (sanitizeName(table["name"]) == table_name) {
			if (reset) {
				let child = $($($("#"+table_name+"-table").children()[1]).children()[1]);
				selectCell(table["name"], table["col-def"][0]["title"], table["col-def"][0]["field"], child.attr("id"));
			} else {
				let elm = $(`#${selected["cell"]}`);
				selectCell(table["name"], current_col["title"], current_col["field"], selected["cell"]);
			}
		}
	}
}

function startEditCell(cell_id) {
	let cell = $(`#${cell_id}`);
	let cell_text = $("#cell-text").val();
	let td = $(`<td id="${cell_id}" class="w3-green text-edit"></td>`);
	let text_area = $(`<textarea id="${cell_id}-edit" onkeydown="finishEditCell(event, '${cell_id}')">${cell_text}</textarea>`);
	text_area.focusout(function (event){finishEditCell(event, cell_id)});
	td.append(text_area);
	cell.parent().css("height", `${cell.outerHeight()}px`);
	cell.replaceWith(td);
	text_area.focus();
}

function startEditHeader(cell_id) {
	let cell = $(`#${cell_id}`);
	let cell_text = $("#col-name").val();
	let th = $(`<th id="${cell_id}" class="w3-green text-edit"></th>`);
	let text_area = $(`<textarea id="${cell_id}-edit" onkeydown="finishEditHeader(event, '${cell_id}')">${cell_text}</textarea>`);
	text_area.focusout(function (event){finishEditHeader(event, cell_id)});
	th.append(text_area);
	cell.parent().css("height", `${cell.outerHeight()}px`);
	cell.replaceWith(th);
	text_area.focus();
}

function finishEditCell(event, cell_id) {
	let cell = $(`#${cell_id}-edit`);
	if (event.ctrlKey && event.key == "Enter") {
		cell.off("focusout");
		let text_area_val = cell.val();
		$("#cell-text").val(text_area_val);
		save();
	} else if (event.key == "Escape") {
		cell.off("focusout");
		let cell_val = $("#cell-text").val();
		cell.val(cell_val);
		save();
	} else if (event.type == "focusout") {
		let text_area_val = cell.val();
		$("#cell-text").val(text_area_val);
		save();
	}
}

function finishEditHeader(event, cell_id) {
	let cell = $(`#${cell_id}-edit`);
	if (event.ctrlKey && event.key == "Enter") {
		cell.off("focusout");
		let text_area_val = $(`#${cell_id}-edit`).val();
		$("#col-name").val(text_area_val);
		save();
	} else if (event.key == "Escape") {
		cell.off("focusout");
		let cell_val = $("#col-name").val();
		$(`#${cell_id}-edit`).val(cell_val);
		save();
	} else if (event.type == "focusout") {
		let text_area_val = $(`#${cell_id}-edit`).val();
		$("#col-name").val(text_area_val);
		save();
	}
}

function saveFile(new_file) {
	ipcRenderer.send("save", data, new_file);
}

function openFile(file_path) {
	$.ajax({
		dataType: "json",
		url: file_path,
		cache: false,
		success: function(json, textStatus, jqXHR ) {
			data = json;
			refreshTables();
			showTable(sanitizeName(data[0]["name"]));
		}
	});
}

ipcRenderer.on("save", (event, new_file) => {
	saveFile(new_file);
})

ipcRenderer.on("open", (event, file_path) => {
	openFile(file_path);
})